<?php

namespace Drupal\restrict_route_by_ip\Service;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\restrict_route_by_ip\Entity\RestrictRouteByIp;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Restrict Ip service.
 */
class RestrictIpService implements RestrictIpInterface {

  /**
   * Drupal ConfigFactory service.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  public $configManager;

  /**
   * Drupal current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * Drupal KillSwitch service.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected $pageCacheKillSwitch;

  /**
   * Drupal logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Drupal RouteMatch service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Drupal EntityStorage on restrict_route entity.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $restrictRouteStorage;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config
   *   Drupal ConfigFactory service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack_service
   *   Drupal RequestStack service.
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $page_cache_kill_switch
   *   Drupal KillSwitch service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   Drupal LoggerChannelFactory service.
   * @param \Drupal\Core\Routing\RouteMatchInterface $current_route_match
   *   Drupal RouteMatch service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Drupal EntityTypeManager service.
   *
   */
  public function __construct(
    ConfigFactory $config,
    RequestStack $request_stack_service,
    KillSwitch $page_cache_kill_switch,
    LoggerChannelFactoryInterface $logger_factory,
    RouteMatchInterface $current_route_match,
    EntityTypeManagerInterface $entity_type_manager) {
    $this->configManager = $config;
    $this->currentRequest = $request_stack_service->getCurrentRequest();
    $this->pageCacheKillSwitch = $page_cache_kill_switch;
    $this->logger = $logger_factory->get('restrict_route_by_ip');
    $this->routeMatch = $current_route_match;
    $this->restrictRouteStorage = $entity_type_manager->getStorage('restrict_route');
  }

  /**
   * {@inheritdoc}
   */
  public function checkRangeIp($ip, $range): bool {
    if (preg_match('/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}(\/\d{1,3})?\z/', $range)) {
      if (strpos($range, '/') === FALSE) {
        $range .= '/32';
      }
      [$range, $netmask] = explode('/', $range, 2);
      $range_decimal = ip2long($range);
      $ip_decimal = ip2long($ip);
      $wildcard_decimal = pow(2, (32 - $netmask)) - 1;
      $netmask_decimal = ~ $wildcard_decimal;
      return (($ip_decimal & $netmask_decimal) == ($range_decimal & $netmask_decimal));
    }
    elseif($ip === $range) {
     // If regex not matching, try string comparison.
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Check if IP is a local IP
   *
   * @param string $ip
   *    An IP.
   *
   * @return bool
   *    TRUE if IP is a local IP, FALSE.
   */
  public function isLocalIp($ip) {
    return in_array($ip, [
      '127.0.0.1',
      '::1',
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function userIpIsRestricted(AccountInterface $account): bool {
    $current_route_name = $this->routeMatch->getRouteName();
    $routes = $this->getAllRestrictedRoutes();

    // Searching for restricted route name.
    $restricted_route_id = array_search($current_route_name, $routes);
    $restricted = FALSE;
    if ($restricted_route_id) {
      // Load the current restricted route entity.
      $restrict_route = RestrictRouteByIp::load($restricted_route_id);
      $current_ip = $this->currentRequest->getClientIp();

      // Check the current IP.
      $ips = $restrict_route->getIps();
      if (is_array($ips) && count($ips) > 0) {
        $logs[] = 'Restricted IPs : ' . json_encode($ips);
        $restricted = TRUE;
        foreach ($ips as $ip) {
          $match = $this->checkRangeIp($current_ip, $ip);
          $logs[] = 'Checked ' . $ip . ', found : ' . strval($match);
          if ($match) {
            $restricted = FALSE;
            break;
          }
        }
      }

      if ($restricted) {
        $logs[] = 'Restricted : ' . $restricted;
        $this->logger->notice($current_ip . ' not accepted.' . implode("\n", $logs));
      }
    }
    return $restricted;
  }

  /**
   * {@inheritdoc}
   */
  public function getAllRestrictedRoutes(): array {
    $routes = [];
     $restrict_route_ids = $this->restrictRouteStorage->getQuery('AND')
       ->condition('status', '1')
       ->execute();
    $entities = RestrictRouteByIp::loadMultiple($restrict_route_ids);

    /** @var \Drupal\restrict_route_by_ip\Entity\RestrictRouteInterface $entity */
    foreach($entities as $entity) {
      $routes[$entity->id()] = $entity->getRoute();
    }
    return $routes;
  }

}
